@@ -18,22 +18,18 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
-#ifndef _HAVE_CLUTTER_PRIVATE_H
-#define _HAVE_CLUTTER_PRIVATE_H
+#ifndef __CLUTTER_PRIVATE_H__
+#define __CLUTTER_PRIVATE_H__
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include <math.h>
 
 #include <glib.h>
@@ -41,6 +37,7 @@
 #include "pango/cogl-pango.h"
 
 #include "clutter-backend.h"
+#include "clutter-device-manager.h"
 #include "clutter-event.h"
 #include "clutter-feature.h"
 #include "clutter-id-pool.h"
@@ -52,46 +49,87 @@
 
 G_BEGIN_DECLS
 
+typedef struct _ClutterMainContext      ClutterMainContext;
+
 typedef enum {
   CLUTTER_ACTOR_UNUSED_FLAG    = 0,
 
   CLUTTER_ACTOR_IN_DESTRUCTION = 1 << 0,
   CLUTTER_ACTOR_IS_TOPLEVEL    = 1 << 1,
   CLUTTER_ACTOR_IN_REPARENT    = 1 << 2,
-  CLUTTER_ACTOR_SYNC_MATRICES  = 1 << 3, /* Used by stage to indicate GL
-					  * viewport / perspective etc
-					  * needs (re)setting.
-                                          */
-  CLUTTER_ACTOR_IN_PAINT       = 1 << 4, /* Used to avoid recursion */
-  CLUTTER_ACTOR_IN_RELAYOUT    = 1 << 5, /* Used to avoid recursion */
-  CLUTTER_STAGE_IN_RESIZE      = 1 << 6
+
+  /* Used by the stage to indicate GL viewport / perspective etc needs
+   * (re)setting.
+   */
+  CLUTTER_ACTOR_SYNC_MATRICES  = 1 << 3,
+
+  /* Used to avoid recursion */
+  CLUTTER_ACTOR_IN_PAINT       = 1 << 4,
+
+  /* Used to avoid recursion */
+  CLUTTER_ACTOR_IN_RELAYOUT    = 1 << 5,
+
+  /* Used by the stage if resizing is an asynchronous operation (like on
+   * X11) to delay queueing relayouts until we got a notification from the
+   * event handling
+   */
+  CLUTTER_STAGE_IN_RESIZE      = 1 << 6,
+
+  /* a flag for internal children of Containers */
+  CLUTTER_ACTOR_INTERNAL_CHILD = 1 << 7
 } ClutterPrivateFlags;
 
 struct _ClutterInputDevice
 {
-  gint          id;
+  GObject parent_instance;
+
+  gint id;
 
   ClutterInputDeviceType device_type;
 
-  ClutterActor *pointer_grab_actor;
-  ClutterActor *motion_last_actor;
+  gchar *device_name;
 
-  gint          click_count;
-  gint          previous_x; 
-  gint          previous_y; 
-  guint32       previous_time;
-  gint          previous_button_number;
+  /* the actor underneath the pointer */
+  ClutterActor *cursor_actor;
+
+  /* the actor that has a grab in place for the device */
+  ClutterActor *pointer_grab_actor;
+
+  /* the current click count */
+  gint click_count;
+
+  /* the stage the device is on */
+  ClutterStage *stage;
+
+  /* the current state */
+  gint current_x;
+  gint current_y;
+  guint32 current_time;
+  gint current_button_number;
+  ClutterModifierType current_state;
+
+  /* the previous state, used for click count generation */
+  gint previous_x;
+  gint previous_y;
+  guint32 previous_time;
+  gint previous_button_number;
+  ClutterModifierType previous_state;
 };
 
-typedef struct _ClutterMainContext ClutterMainContext;
+struct _ClutterStageManager
+{
+  GObject parent_instance;
+
+  GSList *stages;
+};
 
 struct _ClutterMainContext
 {
-  ClutterBackend  *backend;            /* holds a pointer to the windowing 
+  ClutterBackend  *backend;            /* holds a pointer to the windowing
                                           system backend */
   GQueue          *events_queue;       /* the main event queue */
 
-  guint            is_initialized : 1;  
+  guint            is_initialized : 1;
   guint            motion_events_per_actor : 1;/* set for enter/leave events */
   guint            defer_display_setup : 1;
   guint            options_parsed : 1;
@@ -102,8 +140,8 @@ struct _ClutterMainContext
 
   gint             num_reactives;      /* Num of reactive actors */
 
-  ClutterIDPool   *id_pool;            /* mapping between reused integer ids 
-                                        * and actors 
+  ClutterIDPool   *id_pool;            /* mapping between reused integer ids
+                                        * and actors
                                         */
   guint            frame_rate;         /* Default FPS */
 
@@ -111,8 +149,8 @@ struct _ClutterMainContext
                                         * (or NULL if there is no pointer grab
                                         */
   ClutterActor    *keyboard_grab_actor; /* The actor having the pointer grab
-                                         * (or NULL if there is no pointer 
-                                         *  grab) 
+                                         * (or NULL if there is no pointer
+                                         *  grab)
                                          */
   GSList          *shaders;            /* stack of overridden shaders */
 
@@ -125,9 +163,7 @@ struct _ClutterMainContext
   PangoContext     *pango_context;      /* Global Pango context */
   CoglPangoFontMap *font_map;           /* Global font map */
 
-  GSList              *input_devices;   /* For extra input devices, i.e
-                                           MultiTouch */
-
+  ClutterEvent *current_event;
   guint32 last_event_time;
 
   gulong redraw_count;
@@ -154,21 +190,36 @@ PangoContext *_clutter_context_get_pango_context    (ClutterMainContext *self);
 
 #define I_(str)  (g_intern_static_string ((str)))
 
+/* device manager */
+void _clutter_device_manager_add_device     (ClutterDeviceManager *device_manager,
+                                             ClutterInputDevice   *device);
+void _clutter_device_manager_remove_device  (ClutterDeviceManager *device_manager,
+                                             ClutterInputDevice   *device);
+void _clutter_device_manager_update_devices (ClutterDeviceManager *device_manager);
+
+/* input device */
+void          _clutter_input_device_set_coords (ClutterInputDevice  *device,
+                                                gint                 x,
+                                                gint                 y);
+void          _clutter_input_device_set_state  (ClutterInputDevice  *device,
+                                                ClutterModifierType  state);
+void          _clutter_input_device_set_time   (ClutterInputDevice  *device,
+                                                guint32              time_);
+void          _clutter_input_device_set_stage  (ClutterInputDevice  *device,
+                                                ClutterStage        *stage);
+void          _clutter_input_device_set_actor  (ClutterInputDevice  *device,
+                                                ClutterActor        *actor);
+ClutterActor *_clutter_input_device_update     (ClutterInputDevice  *device);
+
 /* stage manager */
-struct _ClutterStageManager
-{
-  GObject parent_instance;
-
-  GSList *stages;
-};
-
-void _clutter_stage_manager_add_stage    (ClutterStageManager *stage_manager,
-                                          ClutterStage        *stage);
-void _clutter_stage_manager_remove_stage (ClutterStageManager *stage_manager,
-                                          ClutterStage        *stage);
+void _clutter_stage_manager_add_stage         (ClutterStageManager *stage_manager,
+                                               ClutterStage        *stage);
+void _clutter_stage_manager_remove_stage      (ClutterStageManager *stage_manager,
+                                               ClutterStage        *stage);
+void _clutter_stage_manager_set_default_stage (ClutterStageManager *stage_manager,
+                                               ClutterStage        *stage);
 
 /* stage */
-
 void                _clutter_stage_set_window           (ClutterStage       *stage,
                                                          ClutterStageWindow *stage_window);
 ClutterStageWindow *_clutter_stage_get_window           (ClutterStage       *stage);
@@ -178,10 +229,17 @@ void                _clutter_stage_maybe_relayout       (ClutterActor       *sta
 gboolean            _clutter_stage_needs_update         (ClutterStage       *stage);
 gboolean            _clutter_stage_do_update            (ClutterStage       *stage);
 
-void     _clutter_stage_queue_event           (ClutterStage *stage,
-					       ClutterEvent *event);
-gboolean _clutter_stage_has_queued_events     (ClutterStage *stage);
-void     _clutter_stage_process_queued_events (ClutterStage *stage);
+
+void     _clutter_stage_queue_event            (ClutterStage *stage,
+					        ClutterEvent *event);
+gboolean _clutter_stage_has_queued_events      (ClutterStage *stage);
+void     _clutter_stage_process_queued_events  (ClutterStage *stage);
+void     _clutter_stage_update_input_devices   (ClutterStage *stage);
+
+int      _clutter_stage_get_pending_swaps      (ClutterStage *stage);
+
+gboolean _clutter_stage_has_full_redraw_queued (ClutterStage *stage);
+
 
 /* vfuncs implemented by backend */
 GType         _clutter_backend_impl_get_type  (void);
@@ -193,6 +251,9 @@ ClutterStageWindow *_clutter_backend_create_stage   (ClutterBackend  *backend,
                                                GError         **error);
 void          _clutter_backend_ensure_context (ClutterBackend  *backend,
                                                ClutterStage    *stage);
+void          _clutter_backend_ensure_context_internal
+                                              (ClutterBackend  *backend,
+                                               ClutterStage    *stage);
 gboolean      _clutter_backend_create_context (ClutterBackend  *backend,
                                                GError         **error);
 
@@ -209,7 +270,7 @@ ClutterFeatureFlags _clutter_backend_get_features (ClutterBackend *backend);
 gfloat        _clutter_backend_get_units_per_em   (ClutterBackend       *backend,
                                                    PangoFontDescription *font_desc);
 
-void          _clutter_feature_init (void);
+gboolean      _clutter_feature_init (GError **error);
 
 /* Reinjecting queued events for processing */
 void _clutter_process_event (ClutterEvent *event);
@@ -252,10 +313,32 @@ void _clutter_actor_set_enable_model_view_transform (ClutterActor *self,
 void _clutter_actor_set_enable_paint_unmapped (ClutterActor *self,
                                                gboolean      enable);
 
+void _clutter_actor_set_has_pointer (ClutterActor *self,
+                                     gboolean      has_pointer);
+
+void _clutter_actor_transform_and_project_box (ClutterActor          *self,
+					       const ClutterActorBox *box,
+					       ClutterVertex          verts[]);
+
+void _clutter_actor_queue_redraw_with_clip (ClutterActor          *self,
+                                            ClutterRedrawFlags     flags,
+                                            ClutterActorBox       *clip);
+const ClutterActorBox *_clutter_actor_get_queue_redraw_clip (ClutterActor *self);
+void _clutter_actor_set_queue_redraw_clip (ClutterActor *self,
+                                           const ClutterActorBox *clip);
+
 void _clutter_run_repaint_functions (void);
 
 gint32 _clutter_backend_get_units_serial (ClutterBackend *backend);
 
+/* XXX - this is going to be a public function in Clutter 1.4 */
+void _clutter_geometry_union (const ClutterGeometry *geometry_a,
+                              const ClutterGeometry *geometry_b,
+                              ClutterGeometry       *result);
+
+/* XXX - this is going to be a public function in Clutter 1.4 */
+gboolean _clutter_actor_has_allocation (ClutterActor *self);
+
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_PRIVATE_H */
